from pyrave.base import BaseRaveAPI
from pyrave.encryption import RaveEncryption
from pyrave.errors import MissingParamError
from pyrave.utils import generate_id


class Payment(BaseRaveAPI):
    """
    Payment API
    """
    def __init__(self):
        super(Payment, self).__init__()
        self.rave_enc = RaveEncryption()
        self.url = ""

    def pay(self, using="card", preauthorised=False, return_encrypted=False, log_url=False, **kwargs):
        """

        :param log_url:
        :param using:
        :param preauthorised:
        :param kwargs:
        :param return_encrypted:
        :return:
        """
        if not kwargs.get("txRef"):
            kwargs["txRef"] = generate_id("txRef")
        if not kwargs.get("device_fingerprint"):
            kwargs["device_fingerprint"] = generate_id("device_fingerprint")
        encrypted_data = self.rave_enc.encrypt(using, preauthorised, **kwargs)
        if return_encrypted:
            return encrypted_data
        url = self.rave_url_map.get("payment_endpoint") + "charge"
        request_data = encrypted_data
        suggested_auth_request = self._exec_request("POST", url, request_data)
        if suggested_auth_request[0] in [400, 401]:
            return suggested_auth_request
        suggested_auth = suggested_auth_request[2].get("suggested_auth")
        if not suggested_auth:
            return suggested_auth_request
        if not kwargs.get("pin"):
                raise MissingParamError("You need to set the pin parameter in the function call "
                                        "to make a payment")
        request_data["suggested_auth"] = suggested_auth
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def get_encrypted_data(self, using="card", preauthorised=False, log_url=False, **kwargs):
        """

        :param log_url:
        :param using:
        :param preauthorised:
        :param kwargs:
        :return:
        """

        return self.rave_enc.encrypt(using, preauthorised, log_url=log_url, **kwargs)

    def validate_charge(self, reference, otp, method="card", log_url=False,):
        """

        :param log_url:
        :param reference:
        :param otp:
        :param method:
        :return:
        """
        request_data = {
            "PBFPubKey": self.public_key,
            "otp": otp
        }
        request_data.update({"transaction_reference": reference}) if method == "card" else request_data.update({"transactionreference": reference})
        url = self.rave_url_map.get("payment_endpoint") + "validatecharge" if method == "card" else self.rave_url_map.get("payment_endpoint") + "validate"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def verify_transaction(self, reference, normalize="1", log_url=False,):
        """

        :param log_url:
        :param reference:
        :param normalize:
        :return:
        """
        request_data = {
            "flw_ref": reference,
            "SECKEY": self.secret_key,
            "normalize": normalize
        }
        url = self.rave_url_map.get("payment_endpoint") + "verify"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def disburse(self, bank_code, account_number, currency, amount, log_url=False,):
        """
        :param log_url:
        :param bank_code:
        :param account_number:
        :param currency:
        :param amount:
        :return:
        """
        request_data = {
                "bank_code": bank_code,
                "account_number": account_number,
                "currency": currency,
                "amount": amount,
                "seckey": self.secret_key
        }
        url = self.rave_url_map.get("disbursement_endpoint")
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def tokenize_charge(self, token, log_url=False, **kwargs):
        """

        :param token:
        :param log_url:
        :param kwargs:
        :return:
        """
        kwargs["SECKEY"] = self.secret_key
        kwargs["token"] = token
        url = self.rave_url_map.get("payment_endpoint") + "tokenized/charge"
        return self._exec_request("POST", url, kwargs, log_url=log_url)

    def refund(self, reference_id, log_url=False,):
        request_data = {
                "ref": reference_id,
                "seckey": self.secret_key,
        }
        url = self.rave_url_map.get("merchant_refund_endpoint")
        return self._exec_request("POST", url, request_data, log_url=log_url)




from pyrave.base import BaseRaveAPI
from pyrave.encryption import RaveEncryption


class Preauth(BaseRaveAPI):
    """
    Preauthorization Class

    """

    def __init__(self):
        super(Preauth, self).__init__()
        self.rave_enc = RaveEncryption()

    def preauthorise_card(self, log_url=False, **kwargs):
        """

        :param log_url:
        :param client:
        :param algo:
        :return:
        """
        encrypted_data = self.rave_enc.encrypt(preauthorised=True, **kwargs)
        if not encrypted_data:
            return encrypted_data
        request_data = {
            "PBFPubKey": self.secret_key,
            "client": encrypted_data[1],
            "algo": encrypted_data[2]
        }
        url = self.rave_url_map.get("payment_endpoint") + "charge"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def capture_preauthorised_transaction(self, transaction_reference, log_url=False):
        """

        :param log_url:
        :param transaction_reference:
        :return:
        """
        request_data = {
            "SECKEY": self.secret_key,
            "flwRef": transaction_reference,
        }
        url = self.rave_url_map.get("payment_endpoint") + "capture"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def refund_or_void_transaction(self, action, reference_id, log_url=False):
        """

        :param log_url:
        :param action:
        :param reference_id:
        :return:
        """
        request_data = {
            "ref": reference_id,
            "action": action,
            "SECKEY": self.secret_key
        }
        url = self.rave_url_map.get("payment_endpoint") + "refundorvoid"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def refund(self, reference_id, log_url=False):
        request_data = {
            "ref": reference_id,
            "seckey": self.secret_key
        }
        url = self.rave_url_map.get("merchant_refund_endpoint")
        return self._exec_request("POST", url, request_data, log_url=log_url)



__version__ = '1.0.1-alpha'
__author__ = "Olamilekan Wahab"
__license__ = 'MIT'
__copyright__ = 'Copyright 2017. Olamilekan Wahab'


from .payment import Payment
from .transaction import Transaction
from .misc import Misc
from .preauth import Preauth
from .encryption import RaveEncryption



class PyRaveError(Exception):
    def __init__(self, message=None, http_status=None):
        super(PyRaveError, self).__init__(message)

        self.message = message
        self.http_status = http_status


class AuthKeyError(PyRaveError):
    """
    Auth Key Not Provided
    """
    pass


class HttpMethodError(PyRaveError):
    """
    Any HTTP related error.
    """
    pass


class MissingParamError(PyRaveError):
    """
    Errors relating to missing parameters
    """
    pass



import base64
from Crypto.Cipher import DES3
import hashlib
from pyrave.base import BaseRaveAPI


class RaveEncryption(BaseRaveAPI):
    """
    Base Encryption Class
    Encrypts User Details and returns pubkey, client and algo
    """
    def __init__(self):
        super(RaveEncryption, self).__init__()

    def encrypt(self, using='card', preauthorised=False, log_url=False, **kwargs):
        """

        :param log_url:
        :param using: Card or Account
        :param preauthorised: Identifies if the transaction is preauthorised
        :param kwargs:
        :return:
        """
        common_params = {
            "seckey": self.secret_key,
            "pubkey": self.public_key,
            "form.PBFPubKey": self.public_key,
            "form.currency": kwargs.get('currency'),
            "form.country": kwargs.get('country'),
            "form.amount": kwargs.get('amount'),
            "form.email": kwargs.get('email'),
            "form.phonenumber": kwargs.get('phonenumber'),
            "form.firstname": kwargs.get('firstname'),
            "form.lastname": kwargs.get('lastname'),
            "form.IP": kwargs.get('IP'),
            "form.txRef": kwargs.get('txRef'),
            "form.device_fingerprint": kwargs.get('device_fingerprint', False)
        }
        card_params = {
            "form.cardno": kwargs.get('cardno'),
            "form.ccv": kwargs.get('ccv', True),
            "form.expirymonth": kwargs.get('expirymonth', True),
            "form.expiryyear": kwargs.get('expiryyear', True),
            "form.charge_type": kwargs.get('charge_type') if preauthorised else preauthorised

        }
        account_params = {
            "form.accountnumber": kwargs.get("accountnumber"),
            "form.accountbank": kwargs.get("accountbank"),
            "form.payment_type": kwargs.get("payment_type", "account")
        }
        url = self.rave_url_map.get("test_encryption_url") if self.implementation == "test" else self.pyrave_encrypt(**kwargs)
        if using == "account":
            account_params.update(common_params)
            return self._exec_request("POST", url, data=account_params)
        card_params.update(common_params)
        return self._exec_request("POST", url, data=card_params, log_url=log_url)

    def pyrave_encrypt(self,  **kwargs):
        hashedseckey = hashlib.md5(self.secret_key.encode("utf-8")).hexdigest()
        hashedseckeylast12 = hashedseckey[-12:]
        seckeyadjusted = self.secret_key.replace('FLWSECK-', '')
        seckeyadjustedfirst12 = seckeyadjusted[:12]
        key = seckeyadjustedfirst12 + hashedseckeylast12
        blockSize = 8
        padDiff = blockSize - (len(kwargs) % blockSize)
        cipher = DES3.new(key, DES3.MODE_ECB)
        plainText = "{}{}".format(kwargs, "".join(chr(padDiff) * padDiff))
        encrypted = base64.b64encode(cipher.encrypt(plainText))
        return encrypted






import string
import random
import uuid


def generate_id(gen_for):
    chars = string.ascii_uppercase + string.digits
    if gen_for == "device_fingerprint":
        return str(uuid.uuid4())
    txref = "{0}_".format(gen_for)
    rand = ''.join(random.choice(chars) for _ in range(20)).lower()
    return txref + rand



from pyrave.base import BaseRaveAPI


class Misc(BaseRaveAPI):
    """
    Base Miscellaneous class
    """
    def __init__(self):
        super(Misc, self).__init__()

    def get_banks(self):
        """

        :return:
        """
        url = self.rave_url_map.get("payment_endpoint") + "flwpbf-banks.js?json=1"
        return self._exec_request('GET', url)

    def get_fee(self, amount, currency, ptype, card6=None):
        """

        :param amount: This is the amount of the product or service to charged from the customer
        :param currency: This is the specified currency to charge the card in.
        :param ptype: This is an optional parameter to be used when the payment type is account payment.
        A value of 2 is to be passed to the endpoint.
        :param card6: This can be used only when the user has entered first 6digits of their card number,
                    it also helps determine international fees on the transaction
                    if the card being used is an international card
        :return:
        """
        request_data = {
            "amount": amount,
            "currency": currency,
            "ptype": ptype,
            "PBFPubKey": self.public_key,
            "card6": card6,
        }
        url = self.rave_url_map.get("payment_endpoint") + "fee"
        return self._exec_request('POST', url, request_data)

    def get_exchange_rates(self, origin_currency, destination_currency, amount=None):
        """

        :param amount:
        :param origin_currency:
        :param destination_currency:
        :return:
        """
        request_data = {
            "origin_currency": origin_currency,
            "destination_currency": destination_currency,
            "SECKEY": self.secret_key
        }
        if amount:
            request_data["amount"] = amount
        url = self.rave_url_map.get("payment_endpoint") + "forex"
        return self._exec_request('POST', url, request_data)



from pyrave.base import BaseRaveAPI


class Transaction(BaseRaveAPI):
    """
    Transaction Classs.
    Handles Transaction Verification and requery.
    Also handles recurrent transaction.
    """
    def __init__(self):
        super(Transaction, self).__init__()

    def verify_transaction(self, normalize="1", flw_ref=None, tx_ref=None, log_url=False):
        """

        :param flw_ref:
        :param log_url:
        :param normalize:
        :param tx_ref:
        :return:
        """
        request_data = {
            "SECKEY": self.secret_key,
            "normalize": normalize
        }
        if tx_ref:
            request_data["tx_ref"] = tx_ref
        if flw_ref:
            request_data["flw_ref"] = flw_ref
        url = self.rave_url_map.get("payment_endpoint") + "verify"
        return self._exec_request("POST", url, request_data, log_url=log_url)

    def verify_transaction_with_xrequery(self, log_url=False, **kwargs):
        """
        :param kwargs:
        :param log_url:
        :return:
        """
        endpoint = self.rave_url_map.get("self.payment_endpoint") + "xrequery"
        url = self._path(endpoint)
        return self._exec_request("POST", url, kwargs, log_url=log_url)

    def get_reccurent_transactions(self, log_url=False):
        """

        :return:
        """
        request_data = {
            "SECKEY": self.secret_key
        }
        url = self.rave_url_map.get("recurring_transaction_endpoint") + "list"
        return self._exec_request("GET", url, request_data, log_url=log_url, params=True)

    def get_reccurrent_transaction(self, transaction_id, log_url=True):
        """

        :param transaction_id:
        :param log_url:
        :return:
        """
        request_data = {
            "SECKEY": self.secret_key,
            "txId": transaction_id
        }
        url = self.rave_url_map.get("recurring_transaction_endpoint") + "list"
        return self._exec_request("GET", url, request_data, log_url=log_url, params=True, )

    def stop_recurrent_transaction(self, transaction_data_id, log_url=False):
        """

        :param transaction_data_id:
        :param log_url:
        :return:
        """
        request_data = {
            "SECKEY": self.secret_key,
            "txId": transaction_data_id
        }
        url = self.rave_url_map.get("recurring_transaction_endpoint") + "stop"

        return self._exec_request("GET", url, request_data, log_url=log_url)







import os
import requests
import json
from pyrave import __version__
from pyrave.errors import AuthKeyError, HttpMethodError


class BaseRaveAPI(object):
    """
        Base PyRave API
    """

    def __init__(self, implementation="test"):
        self.public_key = os.getenv("RAVE_PUBLIC_KEY", None)
        self.secret_key = os.getenv("RAVE_SECRET_KEY", None)
        self.implementation = implementation
        self._base_url = {
            "test": "http://flw-pms-dev.eu-west-1.elasticbeanstalk.com/",
            "live": "https://api.ravepay.co/"
        }
        self.rave_url_map = {
            "test_encryption_url": "https://ravecrypt.herokuapp.com/rave/encrypt",
            "live_encryption_url": "",
            "payment_endpoint": self._base_url.get(self.implementation) + "flwv3-pug/getpaidx/api/",
            "disbursement_endpoint": self._base_url.get(self.implementation) + "merchant/disburse",
            "recurring_transaction_endpoint": self._base_url.get(self.implementation) + "merchant/subscriptions",
            "merchant_refund_endpoint": self._base_url.get(self.implementation) + "gpx/merchant/transactions/refund",
            "docs_url": "https://github.com/Olamyy/pyrave/blob/master/README.md"
        }
        if not self.public_key and not self.secret_key:
            raise AuthKeyError("The secret keys have not been set in your environment. You should get this from your rave "
                               "dashboard and set it in your env. Check {0} for more information".format(self.rave_url_map.get("docs_url")))

        self._content_type = "application/json"

    def _path(self, path):
        url_path = self._base_url.get(self.implementation)
        return url_path + path

    def get_url(self, resource=None):
        """

        :param resource:
        :return:
        """
        return self.rave_url_map.get(resource) if resource else self.rave_url_map

    def http_headers(self):
        return {
            "Content-Type": self._content_type,
            "Authorization": "Bearer " + self.secret_key,
            "user-agent": "pyrave-{}".format(__version__)
        }

    def _json_parser(self, body):
        """Only the status code, the status of the request and the data
        is sent back. the message is irrelevant if ths request was successful"""
        response = body.json()
        status = response.get('status', None)
        message = response.get('message', None)
        data = response.get('data', None)
        if not data or not status or not message:
            return response
        if message:
            return body.status_code, status, data, message
        return body.status_code, status, data

    def _exec_request(self, method, url, data=None, params=False, log_url=False):
        method_map = {
            'GET': requests.get,
            'POST': requests.post,
        }
        payload = json.dumps(data) if data else data
        request = method_map.get(method)

        if not request:
            raise HttpMethodError(
                "Request method not recognised or implemented")
        response = request(
            url, headers=self.http_headers(), data=payload, verify=True) if not params else request(
            url, headers=self.http_headers(), params=payload, verify=True)
        if log_url:
            print("The request URL is {}".format(response.url))
        if response.status_code == 404:
            try:
                if response.json():
                    body = response.json()
                    return response.status_code, body['status'], body['message']
                return response.status_code
            except ValueError or json.decoder.JSONDecodeError:
                return response.status_code, "{} returns a 404.".format(url)
        body = response.json()
        if isinstance(body, list):
            return body
        if body.get('status') == 'error':
            return response.status_code, body
        if response.status_code in [200, 201]:
            return self._json_parser(response)
        response.raise_for_status()



from os import path
import six
from pyrave import __version__, __author__, __license__
from setuptools import setup

here = path.abspath(path.dirname(__file__))

if six.PY2:
    import io
    with io.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

elif six.PY3:
    with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

setup(name='pyrave',
      version=__version__,
      description="Python wrapper flutterwave's rave API",
      long_description=long_description,
      url='https://github.com/Olamyy/pyrave',
      author=__author__,
      author_email='olamyy53@gmail.com',
      license=__license__,
      test_suite='nose.collector',
      tests_require=['nose'],
      install_requires=['requests', 'six', 'pycrypto'],
      download_url='https://github.com/Olamyy/pyrave/archive/0.1.tar.gz',
      packages=['pyrave'],
      keywords=['rave', 'payment', 'pyrave'],
      zip_safe=False
      )
